@@ -1,5 +1,5 @@
 # ----------------------------------------------------------------------------
-# Copyright (c) 2016-2022, QIIME 2 development team.
+# Copyright (c) 2016-2023, QIIME 2 development team.
 #
 # Distributed under the terms of the Modified BSD License.
 #
@@ -8,26 +8,30 @@
 
 import gzip
 import yaml
-import itertools
-import collections
-import collections.abc
+
 import random
 import resource
 import re
+import os
+import warnings
 
 import skbio
 import psutil
+import numpy as np
+import pandas as pd
 
 import qiime2
 from q2_types.per_sample_sequences import (
     SingleLanePerSampleSingleEndFastqDirFmt,
     SingleLanePerSamplePairedEndFastqDirFmt,
     FastqManifestFormat, YamlFormat)
+from q2_types.multiplexed_sequences import ErrorCorrectionDetailsFmt
+from q2_types.feature_data import (
+    BarcodeSequenceFastqIterator, BarcodePairedSequenceFastqIterator
+)
+
 from ._ecc import GolayDecoder
-from ._format import ErrorCorrectionDetailsFmt
-
-
-FastqHeader = collections.namedtuple('FastqHeader', ['id', 'description'])
+from qiime2.util import duplicate
 
 
 class ECDetails:
@@ -56,37 +60,6 @@ class ECDetails:
         self._fp.close()
 
 
-def _read_fastq_seqs(filepath):
-    # This function is adapted from @jairideout's SO post:
-    # http://stackoverflow.com/a/39302117/3424666
-    fh = gzip.open(filepath, 'rt')
-    for seq_header, seq, qual_header, qual in itertools.zip_longest(*[fh] * 4):
-        yield (seq_header.strip(), seq.strip(), qual_header.strip(),
-               qual.strip())
-
-
-def _trim_id(id):
-    return id.rsplit('/', 1)[0]
-
-
-def _trim_description(desc):
-    # The first number of ':' seperated description is the read number
-    if ':' in desc:
-        desc = desc.split(':', 1)[1]
-    return desc.rsplit('/', 1)[0]
-
-
-def _record_to_fastq_header(record):
-    tokens = record[0][1:].split(' ', maxsplit=1)
-    if len(tokens) == 1:
-        id, = tokens
-        description = None
-    else:
-        id, description = tokens
-
-    return FastqHeader(id=id, description=description)
-
-
 # This is global so that it can be tested without changing the actual ulimits.
 # NOTE: UNIX only
 OPEN_FH_LIMIT, _ = resource.getrlimit(resource.RLIMIT_NOFILE)
@@ -126,131 +99,10 @@ def _maintain_open_fh_count(per_sample_fastqs, paired=False):
             rand_fh.close()
 
 
-class BarcodeSequenceFastqIterator(collections.abc.Iterable):
-    def __init__(self, barcode_generator, sequence_generator,
-                 ignore_description_mismatch=False):
-        self.barcode_generator = barcode_generator
-        self.sequence_generator = sequence_generator
-        self.ignore_description_mismatch = ignore_description_mismatch
-
-    def __iter__(self):
-        # Adapted from q2-types
-        for barcode_record, sequence_record in itertools.zip_longest(
-                self.barcode_generator, self.sequence_generator):
-            if barcode_record is None:
-                raise ValueError('More sequences were provided than barcodes.')
-            if sequence_record is None:
-                raise ValueError('More barcodes were provided than sequences.')
-            # The id or description fields may end with "/read-number", which
-            # will differ between the sequence and barcode reads. Confirm that
-            # they are identical up until the last /
-            barcode_header = _record_to_fastq_header(barcode_record)
-            sequence_header = _record_to_fastq_header(sequence_record)
-
-            # confirm that the id fields are equal
-            if _trim_id(barcode_header.id) != \
-               _trim_id(sequence_header.id):
-                raise ValueError(
-                    'Mismatched sequence ids: %s and %s' %
-                    (_trim_id(barcode_header.id),
-                     _trim_id(sequence_header.id)))
-
-            if not self.ignore_description_mismatch:
-                # if a description field is present, confirm that they're equal
-                if barcode_header.description is None and \
-                   sequence_header.description is None:
-                    pass
-                elif barcode_header.description is None:
-                    raise ValueError(
-                        'Barcode header lines do not contain description '
-                        'fields but sequence header lines do.')
-                elif sequence_header.description is None:
-                    raise ValueError(
-                        'Sequence header lines do not contain description '
-                        'fields but barcode header lines do.')
-                elif _trim_description(barcode_header.description) != \
-                        _trim_description(sequence_header.description):
-                    raise ValueError(
-                        'Mismatched sequence descriptions: %s and %s' %
-                        (_trim_description(barcode_header.description),
-                         _trim_description(sequence_header.description)))
-
-            yield barcode_record, sequence_record
-
-
-class BarcodePairedSequenceFastqIterator(collections.abc.Iterable):
-    def __init__(self, barcode_generator, forward_generator,
-                 reverse_generator, ignore_description_mismatch=False):
-        self.barcode_generator = barcode_generator
-        self.forward_generator = forward_generator
-        self.reverse_generator = reverse_generator
-        self.ignore_description_mismatch = ignore_description_mismatch
-
-    def __iter__(self):
-        # Adapted from q2-types
-        for barcode_record, forward_record, reverse_record \
-                in itertools.zip_longest(self.barcode_generator,
-                                         self.forward_generator,
-                                         self.reverse_generator):
-            if barcode_record is None:
-                raise ValueError('More sequences were provided than barcodes.')
-            if forward_record is None:
-                raise ValueError('More barcodes were provided than '
-                                 'forward-sequences.')
-            elif reverse_record is None:
-                raise ValueError('More barcodes were provided than '
-                                 'reverse-sequences.')
-            # The id or description fields may end with "/read-number", which
-            # will differ between the sequence and barcode reads. Confirm that
-            # they are identical up until the last /
-            barcode_header = _record_to_fastq_header(barcode_record)
-            forward_header = _record_to_fastq_header(forward_record)
-            reverse_header = _record_to_fastq_header(reverse_record)
-
-            # confirm that the id fields are equal
-            if not (_trim_id(barcode_header.id) ==
-                    _trim_id(forward_header.id) ==
-                    _trim_id(reverse_header.id)):
-                raise ValueError(
-                    'Mismatched sequence ids: %s, %s, and %s' %
-                    (_trim_id(barcode_header.id),
-                     _trim_id(forward_header.id),
-                     _trim_id(reverse_header.id)))
-
-            if not self.ignore_description_mismatch:
-                # if a description field is present, confirm that they're equal
-                if barcode_header.description is None and \
-                   forward_header.description is None and \
-                   reverse_header.description is None:
-                    pass
-                elif barcode_header.description is None:
-                    raise ValueError(
-                        'Barcode header lines do not contain description '
-                        'fields but sequence header lines do.')
-                elif forward_header.description is None:
-                    raise ValueError(
-                        'Forward-read header lines do not contain description '
-                        'fields but barcode header lines do.')
-                elif reverse_header.description is None:
-                    raise ValueError(
-                        'Reverse-read header lines do not contain description '
-                        'fields but barcode header lines do.')
-                elif not (_trim_description(barcode_header.description) ==
-                          _trim_description(forward_header.description) ==
-                          _trim_description(reverse_header.description)):
-                    raise ValueError(
-                        'Mismatched sequence descriptions: %s, %s, and %s' %
-                        (_trim_description(barcode_header.description),
-                         _trim_description(forward_header.description),
-                         _trim_description(reverse_header.description)))
-
-            yield barcode_record, forward_record, reverse_record
-
-
 def _make_barcode_map(barcodes, rev_comp_mapping_barcodes):
     barcode_map = {}
     barcode_len = None
-    for sample_id, barcode in barcodes.to_series().iteritems():
+    for sample_id, barcode in barcodes.to_series().items():
         if barcode_len is None:
             barcode_len = len(barcode)
         elif len(barcode) != barcode_len:
@@ -504,3 +356,99 @@ def emp_paired(seqs: BarcodePairedSequenceFastqIterator,
     _write_metadata_yaml(result)
 
     return result, ec_details_fmt
+
+
+def partition_samples_single(demux: SingleLanePerSampleSingleEndFastqDirFmt,
+                             num_partitions: int = None
+                             ) -> SingleLanePerSampleSingleEndFastqDirFmt:
+    return _partition_helper(demux, num_partitions, paired=False)
+
+
+def partition_samples_paired(demux: SingleLanePerSamplePairedEndFastqDirFmt,
+                             num_partitions: int = None
+                             ) -> SingleLanePerSamplePairedEndFastqDirFmt:
+    return _partition_helper(demux, num_partitions, paired=True)
+
+
+def _partition_helper(demux, num_partitions, paired):
+    """ Deal with partitioning logic that is largely the same regardless of
+        single or paired.
+    """
+    # Adjust based on if we are in the single or paired end case
+    result_class = type(demux)
+
+    partitioned_demux = {}
+    df = demux.manifest.view(pd.DataFrame)
+
+    # Make sure we are partitioning on samples if no number of partitions or
+    # too many partitions specified and warn if they specified too many
+    # partitions
+    num_samples = df.shape[0]
+    if num_partitions is None:
+        num_partitions = num_samples
+    elif num_partitions > num_samples:
+        warnings.warn("You have requested a number of partitions"
+                      f" '{num_partitions}' that is greater than your number"
+                      f" of samples '{num_samples}.' Your data will be"
+                      f" partitioned by sample into '{num_samples}'"
+                      " partitions.")
+        num_partitions = num_samples
+
+    partitioned_df = np.array_split(df, num_partitions)
+    for i, _df in enumerate(partitioned_df, 1):
+        result = result_class()
+
+        manifest_string = ''
+        for sample in _df.iterrows():
+            sample_id = sample[0]
+
+            manifest_string += _partition_duplicate(
+                    sample, sample_id, result, 'forward')
+            if paired:
+                manifest_string += _partition_duplicate(
+                    sample, sample_id, result, 'reverse')
+
+        manifest = _partition_write_manifest(manifest_string, paired)
+        result.manifest.write_data(manifest, FastqManifestFormat)
+        _write_metadata_yaml(result)
+
+        # If we have one sample per partition we name the partitions after the
+        # samples. Otherwise we number them
+        if num_partitions == num_samples:
+            partitioned_demux[sample_id] = result
+        else:
+            partitioned_demux[i] = result
+
+    return partitioned_demux
+
+
+def _partition_duplicate(sample, sample_id, result, direction):
+    """ Duplicate the given direction of the sample into the result and return
+        the corresponding line for the manifest.
+    """
+    in_path = sample[1][direction]
+
+    artifact_name = os.path.basename(in_path)
+    out_path = os.path.join(result.path, artifact_name)
+    duplicate(in_path, out_path)
+
+    return '%s,%s,%s\n' % (sample_id, artifact_name, direction)
+
+
+def _partition_write_manifest(manifest_string, paired):
+    """ Add header to manifest then write to file.
+    """
+    manifest = FastqManifestFormat()
+
+    header_string = 'sample-id,filename,direction\n'
+    if not paired:
+        header_string += \
+            ('# direction is not meaningful in this file as these\n'
+             '# data may be derived from forward, reverse, or \n'
+             '# joined reads\n')
+    manifest_string = header_string + manifest_string
+
+    with manifest.open() as manifest_fh:
+        manifest_fh.write(manifest_string)
+
+    return manifest
