@@ -1,5 +1,5 @@
 # ----------------------------------------------------------------------------
-# Copyright (c) 2016-2022, QIIME 2 development team.
+# Copyright (c) 2016-2023, QIIME 2 development team.
 #
 # Distributed under the terms of the Modified BSD License.
 #
@@ -17,7 +17,7 @@ import pandas as pd
 import seaborn as sns
 import numpy as np
 
-from q2_demux._demux import _read_fastq_seqs
+from q2_types.feature_data._util import (_read_fastq_seqs, _PlotQualView)
 import q2templates
 
 TEMPLATES = pkg_resources.resource_filename('q2_demux', '_summarize')
@@ -29,17 +29,6 @@ def _decode_qual_to_phred33(qual_str):
     return qual
 
 
-# TODO: Remove _PlotQualView once QIIME 2 #220 completed
-class _PlotQualView:
-    """
-    A very simple pass-through view which is made up of a single-end or
-    paired-end directory format with a bool indicating if single or paired.
-    """
-    def __init__(self, directory_format, paired):
-        self.directory_format = directory_format
-        self.paired = paired
-
-
 def _link_sample_n_to_file(file_records, counts, subsample_ns, direction):
     results = collections.defaultdict(list)
     for num in subsample_ns:
@@ -133,7 +122,7 @@ def summarize(output_dir: str, data: _PlotQualView, n: int = 10000) -> None:
 
             # If we have an empty direction for a sample that will be a nan in
             # the manifest. Skip that nan
-            if type(filename) != str:
+            if type(filename) is not str:
                 if filename is None or np.isnan(filename):
                     continue
 
@@ -188,7 +177,7 @@ def summarize(output_dir: str, data: _PlotQualView, n: int = 10000) -> None:
                             result.max(), sequence_count[direction]]],
                           index=['%s reads' % (direction,)],
                           columns=summary_columns)
-        context['result_data'] = context['result_data'].append(df)
+        context['result_data'] = pd.concat([context['result_data'], df])
 
         html_df = result.to_frame()
         context['result'] = context['result'].join(html_df, how='outer')
