@@ -1,30 +1,29 @@
 # ----------------------------------------------------------------------------
-# Copyright (c) 2016-2022, QIIME 2 development team.
+# Copyright (c) 2016-2023, QIIME 2 development team.
 #
 # Distributed under the terms of the Modified BSD License.
 #
 # The full license is in the file LICENSE, distributed with this software.
 # ----------------------------------------------------------------------------
 
-import importlib
 
 from qiime2.plugin import (
     Plugin, Metadata, MetadataColumn, Categorical, Bool, Str, Int, Float,
-    Range, Citations, TypeMatch
+    List, Collection, Range, Citations, TypeMatch
 )
 
 from q2_types.sample_data import SampleData
+from q2_types.metadata import ImmutableMetadata
 from q2_types.per_sample_sequences import (
     SequencesWithQuality, PairedEndSequencesWithQuality,
     JoinedSequencesWithQuality)
 
-import q2_demux
-from ._type import (RawSequences, EMPSingleEndSequences, EMPPairedEndSequences,
-                    ErrorCorrectionDetails)
-from ._format import (EMPMultiplexedDirFmt, ErrorCorrectionDetailsDirFmt,
-                      EMPSingleEndDirFmt, EMPSingleEndCasavaDirFmt,
-                      EMPPairedEndDirFmt, EMPPairedEndCasavaDirFmt)
+from q2_types.multiplexed_sequences import (RawSequences,
+                                            EMPSingleEndSequences,
+                                            EMPPairedEndSequences,
+                                            ErrorCorrectionDetails)
 
+import q2_demux
 import q2_demux._examples as ex
 
 citations = Citations.load('citations.bib', package='q2_demux')
@@ -40,34 +39,6 @@ plugin = Plugin(
     short_description='Plugin for demultiplexing & viewing sequence quality.'
 )
 
-plugin.register_semantic_types(
-    RawSequences, EMPSingleEndSequences, EMPPairedEndSequences,
-    ErrorCorrectionDetails)
-
-plugin.register_formats(EMPMultiplexedDirFmt, ErrorCorrectionDetailsDirFmt,
-                        EMPSingleEndDirFmt, EMPSingleEndCasavaDirFmt,
-                        EMPPairedEndDirFmt, EMPPairedEndCasavaDirFmt)
-
-# TODO: remove when aliasing exists
-plugin.register_semantic_type_to_format(
-    RawSequences,
-    artifact_format=EMPSingleEndDirFmt
-)
-
-plugin.register_semantic_type_to_format(
-    EMPSingleEndSequences,
-    artifact_format=EMPSingleEndDirFmt
-)
-
-plugin.register_semantic_type_to_format(
-    EMPPairedEndSequences,
-    artifact_format=EMPPairedEndDirFmt
-)
-
-plugin.register_semantic_type_to_format(
-    ErrorCorrectionDetails,
-    artifact_format=ErrorCorrectionDetailsDirFmt
-)
 
 plugin.methods.register_function(
     function=q2_demux.emp_single,
@@ -160,6 +131,55 @@ plugin.methods.register_function(
         citations['hamady2009']]
 )
 
+demux_description = 'The demultiplexed sequences to partition.'
+num_partitions_description = 'The number of partitions to split the' \
+                             ' demultiplexed sequences into. Defaults to' \
+                             ' partitioning into individual samples.'
+partitioned_demux_description = 'The partitioned demultiplexed sequences.'
+
+plugin.methods.register_function(
+    function=q2_demux.partition_samples_single,
+    inputs={'demux': SampleData[SequencesWithQuality]},
+    parameters={'num_partitions': Int % Range(1, None)},
+    outputs=[
+        ('partitioned_demux', Collection[SampleData[SequencesWithQuality]]),
+    ],
+    input_descriptions={
+        'demux': demux_description
+    },
+    parameter_descriptions={
+        'num_partitions': num_partitions_description
+    },
+    output_descriptions={
+        'partitioned_demux': partitioned_demux_description
+    },
+    name='Split demultiplexed sequence data into partitions.',
+    description=('Partition demultiplexed single end sequences into '
+                 'individual samples or the number of partitions specified.'),
+)
+
+plugin.methods.register_function(
+    function=q2_demux.partition_samples_paired,
+    inputs={'demux': SampleData[PairedEndSequencesWithQuality]},
+    parameters={'num_partitions': Int % Range(1, None)},
+    outputs=[
+        ('partitioned_demux',
+         Collection[SampleData[PairedEndSequencesWithQuality]]),
+    ],
+    input_descriptions={
+        'demux': demux_description
+    },
+    parameter_descriptions={
+        'num_partitions': num_partitions_description
+    },
+    output_descriptions={
+        'partitioned_demux': partitioned_demux_description
+    },
+    name='Split demultiplexed sequence data into partitions.',
+    description=('Partition demultiplexed paired end sequences into '
+                 'individual samples or the number of partitions specified.'),
+)
+
 plugin.visualizers.register_function(
     function=q2_demux.summarize,
     inputs={'data':
@@ -185,6 +205,25 @@ plugin.visualizers.register_function(
     examples={'demux': ex.summarize}
 )
 
+plugin.methods.register_function(
+    function=q2_demux.tabulate_read_counts,
+    inputs={'sequences':
+            List[SampleData[SequencesWithQuality |
+                            PairedEndSequencesWithQuality |
+                            JoinedSequencesWithQuality]]},
+    parameters={},
+    outputs=[
+        ('counts', ImmutableMetadata)
+    ],
+    input_descriptions={
+        'sequences': 'One or more collections of demultiplexed sequences.'
+    },
+    parameter_descriptions={},
+    name='Tabulate counts per sample',
+    description=('Generate a per-sample count of sequence reads.'),
+    examples={}
+)
+
 plugin.methods.register_function(
     function=q2_demux.subsample_single,
     inputs={'sequences': SampleData[SequencesWithQuality |
@@ -278,5 +317,3 @@ plugin.methods.register_function(
                 'with the WHERE clause, and the `exclude_ids` parameter '
                 'allows for filtering of all samples not specified.',
 )
-
-importlib.import_module('q2_demux._transformer')
