@@ -1,5 +1,5 @@
 # ----------------------------------------------------------------------------
-# Copyright (c) 2016-2022, QIIME 2 development team.
+# Copyright (c) 2016-2023, QIIME 2 development team.
 #
 # Distributed under the terms of the Modified BSD License.
 #
@@ -19,10 +19,13 @@ import skbio
 import qiime2
 import numpy.testing as npt
 
-from qiime2.plugin.testing import TestPluginBase
-from q2_demux._demux import (BarcodeSequenceFastqIterator,
-                             BarcodePairedSequenceFastqIterator)
-from q2_demux import emp_single, emp_paired, summarize
+from q2_types.feature_data._transformer import (
+    BarcodeSequenceFastqIterator,
+    BarcodePairedSequenceFastqIterator)
+from qiime2.plugin.testing import TestPluginBase, assert_no_nans_in_tables
+
+from q2_demux import (emp_single, emp_paired, partition_samples_single,
+                      partition_samples_paired, summarize)
 from q2_types.per_sample_sequences import (
     FastqGzFormat, FastqManifestFormat,
     SingleLanePerSampleSingleEndFastqDirFmt,
@@ -38,161 +41,6 @@ class TestBase(TestPluginBase):
         self.execute_examples()
 
 
-class BarcodeSequenceFastqIteratorTests(unittest.TestCase):
-
-    def test_valid(self):
-        barcodes = [('@s1/2 abc/2', 'AAAA', '+', 'YYYY'),
-                    ('@s2/2 abc/2', 'AAAA', '+', 'PPPP'),
-                    ('@s3/2 abc/2', 'AACC', '+', 'PPPP'),
-                    ('@s4/2 abc/2', 'AACC', '+', 'PPPP')]
-
-        sequences = [('@s1/1 abc/1', 'GGG', '+', 'YYY'),
-                     ('@s2/1 abc/1', 'CCC', '+', 'PPP'),
-                     ('@s3/1 abc/1', 'AAA', '+', 'PPP'),
-                     ('@s4/1 abc/1', 'TTT', '+', 'PPP')]
-
-        bsi = BarcodeSequenceFastqIterator(barcodes, sequences)
-        for i, (barcode, sequence) in enumerate(bsi):
-            self.assertEqual(barcode, barcodes[i])
-            self.assertEqual(sequence, sequences[i])
-
-    def test_too_few_barcodes(self):
-        barcodes = [('@s1/2 abc/2', 'AAAA', '+', 'YYYY'),
-                    ('@s2/2 abc/2', 'AAAA', '+', 'PPPP'),
-                    ('@s3/2 abc/2', 'AACC', '+', 'PPPP')]
-
-        sequences = [('@s1/1 abc/1', 'GGG', '+', 'YYY'),
-                     ('@s2/1 abc/1', 'CCC', '+', 'PPP'),
-                     ('@s3/1 abc/1', 'AAA', '+', 'PPP'),
-                     ('@s4/1 abc/1', 'TTT', '+', 'PPP')]
-
-        bsi = BarcodeSequenceFastqIterator(barcodes, sequences)
-        with self.assertRaises(ValueError):
-            list(bsi)
-
-    def test_too_few_sequences(self):
-        barcodes = [('@s1/2 abc/2', 'AAAA', '+', 'YYYY'),
-                    ('@s2/2 abc/2', 'AAAA', '+', 'PPPP'),
-                    ('@s3/2 abc/2', 'AACC', '+', 'PPPP'),
-                    ('@s4/2 abc/2', 'AACC', '+', 'PPPP')]
-
-        sequences = [('@s1/1 abc/1', 'GGG', '+', 'YYY')]
-
-        bsi = BarcodeSequenceFastqIterator(barcodes, sequences)
-        with self.assertRaises(ValueError):
-            list(bsi)
-
-    def test_mismatched_id(self):
-        barcodes = [('@s1/2 abc/2', 'AAAA', '+', 'YYYY'),
-                    ('@s2/2 abc/2', 'AAAA', '+', 'PPPP'),
-                    ('@s3/2 abc/2', 'AACC', '+', 'PPPP'),
-                    ('@s4/2 abc/2', 'AACC', '+', 'PPPP')]
-
-        sequences = [('@s1/1 abc/1', 'GGG', '+', 'YYY'),
-                     ('@s2/1 abc/1', 'CCC', '+', 'PPP'),
-                     ('@s3/1 abc/1', 'AAA', '+', 'PPP'),
-                     ('@s5/1 abc/1', 'TTT', '+', 'PPP')]
-
-        bsi = BarcodeSequenceFastqIterator(barcodes, sequences)
-        with self.assertRaises(ValueError):
-            list(bsi)
-
-    def test_mismatched_description(self):
-        barcodes = [('@s1/2 abc/2', 'AAAA', '+', 'YYYY'),
-                    ('@s2/2 abc/2', 'AAAA', '+', 'PPPP'),
-                    ('@s3/2 abc/2', 'AACC', '+', 'PPPP'),
-                    ('@s4/2 abc/2', 'AACC', '+', 'PPPP')]
-
-        sequences = [('@s1/1 abc/1', 'GGG', '+', 'YYY'),
-                     ('@s2/1 abc/1', 'CCC', '+', 'PPP'),
-                     ('@s3/1 abc/1', 'AAA', '+', 'PPP'),
-                     ('@s4/1 abd/1', 'TTT', '+', 'PPP')]
-
-        bsi = BarcodeSequenceFastqIterator(barcodes, sequences)
-        with self.assertRaises(ValueError):
-            list(bsi)
-
-    def test_mismatch_description_override(self):
-        barcodes = [('@s1/2 abc/2', 'AAAA', '+', 'YYYY'),
-                    ('@s2/2 abc/2', 'AAAA', '+', 'PPPP'),
-                    ('@s3/2 abc/2', 'AACC', '+', 'PPPP'),
-                    ('@s4/2 abc/2', 'AACC', '+', 'PPPP')]
-
-        sequences = [('@s1/1 abc/1', 'GGG', '+', 'YYY'),
-                     ('@s2/1 abc/1', 'CCC', '+', 'PPP'),
-                     ('@s3/1 abc/1', 'AAA', '+', 'PPP'),
-                     ('@s4/1 abd/1', 'TTT', '+', 'PPP')]
-
-        bsi = BarcodeSequenceFastqIterator(barcodes, sequences,
-                                           ignore_description_mismatch=True)
-        self.assertEqual(len(list(bsi)), 4)
-
-    def test_mismatched_handles_slashes_in_id(self):
-        # mismatch is detected as being before the last slash, even if there
-        # is more than one slash
-        barcodes = [('@s1/2/2 abc/2', 'AAAA', '+', 'YYYY')]
-        sequences = [('@s1/1/1 abc/1', 'GGG', '+', 'YYY')]
-
-        bsi = BarcodeSequenceFastqIterator(barcodes, sequences)
-        with self.assertRaises(ValueError):
-            list(bsi)
-
-    def test_mismatched_handles_slashes_in_description(self):
-        # mismatch is detected as being before the last slash, even if there
-        # is more than one slash
-        barcodes = [('@s1/2 a/2/2', 'AAAA', '+', 'YYYY')]
-        sequences = [('@s1/1 a/1/1', 'GGG', '+', 'YYY')]
-
-        bsi = BarcodeSequenceFastqIterator(barcodes, sequences)
-        with self.assertRaises(ValueError):
-            list(bsi)
-
-    def test_no_description(self):
-        barcodes = [('@s1/2', 'AAAA', '+', 'YYYY'),
-                    ('@s2/2', 'AAAA', '+', 'PPPP'),
-                    ('@s3/2', 'AACC', '+', 'PPPP'),
-                    ('@s4/2', 'AACC', '+', 'PPPP')]
-
-        sequences = [('@s1/1', 'GGG', '+', 'YYY'),
-                     ('@s2/1', 'CCC', '+', 'PPP'),
-                     ('@s3/1', 'AAA', '+', 'PPP'),
-                     ('@s4/1', 'TTT', '+', 'PPP')]
-
-        bsi = BarcodeSequenceFastqIterator(barcodes, sequences)
-        for i, (barcode, sequence) in enumerate(bsi):
-            self.assertEqual(barcode, barcodes[i])
-            self.assertEqual(sequence, sequences[i])
-
-    def test_only_one_description(self):
-        barcodes = [('@s1/2 abc', 'AAAA', '+', 'YYYY'),
-                    ('@s2/2 abc', 'AAAA', '+', 'PPPP'),
-                    ('@s3/2 abc', 'AACC', '+', 'PPPP'),
-                    ('@s4/2 abc', 'AACC', '+', 'PPPP')]
-
-        sequences = [('@s1/1', 'GGG', '+', 'YYY'),
-                     ('@s2/1', 'CCC', '+', 'PPP'),
-                     ('@s3/1', 'AAA', '+', 'PPP'),
-                     ('@s4/1', 'TTT', '+', 'PPP')]
-
-        bsi = BarcodeSequenceFastqIterator(barcodes, sequences)
-        with self.assertRaises(ValueError):
-            list(bsi)
-
-        barcodes = [('@s1/2', 'AAAA', '+', 'YYYY'),
-                    ('@s2/2', 'AAAA', '+', 'PPPP'),
-                    ('@s3/2', 'AACC', '+', 'PPPP'),
-                    ('@s4/2', 'AACC', '+', 'PPPP')]
-
-        sequences = [('@s1/1 abc', 'GGG', '+', 'YYY'),
-                     ('@s2/1 abc', 'CCC', '+', 'PPP'),
-                     ('@s3/1 abc', 'AAA', '+', 'PPP'),
-                     ('@s4/1 abc', 'TTT', '+', 'PPP')]
-
-        bsi = BarcodeSequenceFastqIterator(barcodes, sequences)
-        with self.assertRaises(ValueError):
-            list(bsi)
-
-
 class EmpTestingUtils:
     def _compare_sequence_to_record(self, sequence, fields):
         header_line = ' '.join([sequence.metadata['id'],
@@ -598,6 +446,105 @@ class EmpSingleTests(unittest.TestCase, EmpTestingUtils):
                 golay_error_correction=False,
             )
 
+    def test_partition(self):
+        demux, _ = emp_single(self.bsi, self.barcode_map,
+                              golay_error_correction=False)
+
+        partition = partition_samples_single(demux)
+
+        exp_samples = ('sample1_1_L001_R1_001.fastq.gz',
+                       'sample2_3_L001_R1_001.fastq.gz',
+                       'sample3_2_L001_R1_001.fastq.gz',
+                       'sample4_5_L001_R1_001.fastq.gz',
+                       'sample5_4_L001_R1_001.fastq.gz')
+        exp_indices = ([0, 5], [2, 4], [1, 3], [7, 10], [6, 8, 9])
+
+        for idx, (id, sample) in enumerate(partition.items()):
+            self.assertEqual(id, f'sample{idx + 1}')
+
+            act_manifest = \
+                list(sample.manifest.view(FastqManifestFormat).open())
+            exp_manifest = ['sample-id,filename,direction\n',
+                            f'sample{idx + 1},{exp_samples[idx]},forward\n']
+            self._compare_manifests(act_manifest, exp_manifest)
+
+            output_fastq = list(sample.sequences.iter_views(FastqGzFormat))
+            self.assertEqual(len(output_fastq), 1)
+
+            self._validate_sample_fastq(
+                output_fastq[0][1].open(), self.sequences, exp_indices[idx])
+
+    def test_partition_num_specified(self):
+        demux, _ = emp_single(self.bsi, self.barcode_map,
+                              golay_error_correction=False)
+        partition = partition_samples_single(demux, 2)
+        exp_indices = ([0, 5], [2, 4], [1, 3], [7, 10], [6, 8, 9])
+
+        sample = partition[1]
+        act_manifest = list(sample.manifest.view(FastqManifestFormat).open())
+
+        exp_manifest = ['sample-id,filename,direction\n',
+                        'sample1,sample1_1_L001_R1_001.fastq.gz,forward\n',
+                        'sample2,sample2_3_L001_R1_001.fastq.gz,forward\n',
+                        'sample3,sample3_2_L001_R1_001.fastq.gz,forward\n']
+        self._compare_manifests(act_manifest, exp_manifest)
+
+        output_fastq = list(sample.sequences.iter_views(FastqGzFormat))
+        self.assertEqual(len(output_fastq), 3)
+
+        self._validate_sample_fastq(
+            output_fastq[0][1].open(), self.sequences, exp_indices[0])
+        self._validate_sample_fastq(
+            output_fastq[1][1].open(), self.sequences, exp_indices[1])
+        self._validate_sample_fastq(
+            output_fastq[2][1].open(), self.sequences, exp_indices[2])
+
+        sample = partition[2]
+        act_manifest = list(sample.manifest.view(FastqManifestFormat).open())
+
+        exp_manifest = ['sample-id,filename,direction\n',
+                        'sample4,sample4_5_L001_R1_001.fastq.gz,forward\n',
+                        'sample5,sample5_4_L001_R1_001.fastq.gz,forward\n']
+        self._compare_manifests(act_manifest, exp_manifest)
+
+        output_fastq = list(sample.sequences.iter_views(FastqGzFormat))
+        self.assertEqual(len(output_fastq), 2)
+
+        self._validate_sample_fastq(
+            output_fastq[0][1].open(), self.sequences, exp_indices[3])
+        self._validate_sample_fastq(
+            output_fastq[1][1].open(), self.sequences, exp_indices[4])
+
+    def test_partition_more_partitions_than_samples(self):
+        demux, _ = emp_single(self.bsi, self.barcode_map,
+                              golay_error_correction=False)
+
+        with self.assertWarnsRegex(
+                UserWarning, "You have requested a number of.*100.*5.*5"):
+            partition = partition_samples_single(demux, 100)
+
+        exp_samples = ('sample1_1_L001_R1_001.fastq.gz',
+                       'sample2_3_L001_R1_001.fastq.gz',
+                       'sample3_2_L001_R1_001.fastq.gz',
+                       'sample4_5_L001_R1_001.fastq.gz',
+                       'sample5_4_L001_R1_001.fastq.gz')
+        exp_indices = ([0, 5], [2, 4], [1, 3], [7, 10], [6, 8, 9])
+
+        for idx, (id, sample) in enumerate(partition.items()):
+            self.assertEqual(id, f'sample{idx + 1}')
+
+            act_manifest = \
+                list(sample.manifest.view(FastqManifestFormat).open())
+            exp_manifest = ['sample-id,filename,direction\n',
+                            f'sample{idx + 1},{exp_samples[idx]},forward\n']
+            self._compare_manifests(act_manifest, exp_manifest)
+
+            output_fastq = list(sample.sequences.iter_views(FastqGzFormat))
+            self.assertEqual(len(output_fastq), 1)
+
+            self._validate_sample_fastq(
+                output_fastq[0][1].open(), self.sequences, exp_indices[idx])
+
 
 class EmpPairedTests(unittest.TestCase, EmpTestingUtils):
     def setUp(self):
@@ -947,6 +894,176 @@ class EmpPairedTests(unittest.TestCase, EmpTestingUtils):
         self.check_valid(bpsi, self.barcode_map, golay_error_correction=False,
                          rev_comp_barcodes=True)
 
+    def test_partition(self):
+        demux, _ = emp_paired(self.bpsi, self.barcode_map,
+                              golay_error_correction=False)
+
+        partition = partition_samples_paired(demux)
+
+        exp_samples_fwd = ('sample1_1_L001_R1_001.fastq.gz',
+                           'sample2_3_L001_R1_001.fastq.gz',
+                           'sample3_2_L001_R1_001.fastq.gz',
+                           'sample4_5_L001_R1_001.fastq.gz',
+                           'sample5_4_L001_R1_001.fastq.gz')
+        exp_samples_rev = ('sample1_1_L001_R2_001.fastq.gz',
+                           'sample2_3_L001_R2_001.fastq.gz',
+                           'sample3_2_L001_R2_001.fastq.gz',
+                           'sample4_5_L001_R2_001.fastq.gz',
+                           'sample5_4_L001_R2_001.fastq.gz')
+        exp_indices = ([0, 5], [2, 4], [1, 3], [7, 10], [6, 8, 9])
+
+        for idx, (id, sample) in enumerate(partition.items()):
+            self.assertEqual(id, f'sample{idx + 1}')
+
+            act_manifest = \
+                list(sample.manifest.view(FastqManifestFormat).open())
+            exp_manifest = \
+                ['sample-id,filename,direction\n',
+                 f'sample{idx + 1},{exp_samples_fwd[idx]},forward\n',
+                 f'sample{idx + 1},{exp_samples_rev[idx]},reverse\n']
+            self._compare_manifests(act_manifest, exp_manifest)
+
+            forward_fastq = [
+                view for path, view in
+                sample.sequences.iter_views(FastqGzFormat)
+                if 'R1_001.fastq' in path.name]
+            self.assertEqual(len(forward_fastq), 1)
+
+            reverse_fastq = [
+                view for path, view in
+                sample.sequences.iter_views(FastqGzFormat)
+                if 'R2_001.fastq' in path.name]
+            self.assertEqual(len(reverse_fastq), 1)
+
+            self._validate_sample_fastq(
+                forward_fastq[0].open(), self.forward, exp_indices[idx])
+            self._validate_sample_fastq(
+                reverse_fastq[0].open(), self.reverse, exp_indices[idx])
+
+    def test_partition_num_specified(self):
+        demux, _ = emp_paired(self.bpsi, self.barcode_map,
+                              golay_error_correction=False)
+        partition = partition_samples_paired(demux, 2)
+        exp_indices = ([0, 5], [2, 4], [1, 3], [7, 10], [6, 8, 9])
+
+        sample = partition[1]
+        act_manifest = list(sample.manifest.view(FastqManifestFormat).open())
+
+        exp_manifest = ['sample-id,filename,direction\n',
+                        'sample1,sample1_1_L001_R1_001.fastq.gz,forward\n',
+                        'sample1,sample1_1_L001_R2_001.fastq.gz,reverse\n',
+                        'sample2,sample2_3_L001_R1_001.fastq.gz,forward\n',
+                        'sample2,sample2_3_L001_R2_001.fastq.gz,reverse\n',
+                        'sample3,sample3_2_L001_R1_001.fastq.gz,forward\n',
+                        'sample3,sample3_2_L001_R2_001.fastq.gz,reverse\n']
+        self._compare_manifests(act_manifest, exp_manifest)
+
+        forward_fastq = [
+            view for path, view in
+            sample.sequences.iter_views(FastqGzFormat)
+            if 'R1_001.fastq' in path.name]
+        self.assertEqual(len(forward_fastq), 3)
+
+        self._validate_sample_fastq(
+            forward_fastq[0].open(), self.forward, exp_indices[0])
+        self._validate_sample_fastq(
+            forward_fastq[1].open(), self.forward, exp_indices[1])
+        self._validate_sample_fastq(
+            forward_fastq[2].open(), self.forward, exp_indices[2])
+
+        reverse_fastq = [
+            view for path, view in
+            sample.sequences.iter_views(FastqGzFormat)
+            if 'R2_001.fastq' in path.name]
+        self.assertEqual(len(reverse_fastq), 3)
+
+        self._validate_sample_fastq(
+            reverse_fastq[0].open(), self.reverse, exp_indices[0])
+        self._validate_sample_fastq(
+            reverse_fastq[1].open(), self.reverse, exp_indices[1])
+        self._validate_sample_fastq(
+            reverse_fastq[2].open(), self.reverse, exp_indices[2])
+
+        sample = partition[2]
+        act_manifest = list(sample.manifest.view(FastqManifestFormat).open())
+
+        exp_manifest = ['sample-id,filename,direction\n',
+                        'sample4,sample4_5_L001_R1_001.fastq.gz,forward\n',
+                        'sample4,sample4_5_L001_R2_001.fastq.gz,reverse\n',
+                        'sample5,sample5_4_L001_R1_001.fastq.gz,forward\n',
+                        'sample5,sample5_4_L001_R2_001.fastq.gz,reverse\n']
+        self._compare_manifests(act_manifest, exp_manifest)
+
+        forward_fastq = [
+            view for path, view in
+            sample.sequences.iter_views(FastqGzFormat)
+            if 'R1_001.fastq' in path.name]
+        self.assertEqual(len(forward_fastq), 2)
+
+        self._validate_sample_fastq(
+            forward_fastq[0].open(), self.forward, exp_indices[3])
+        self._validate_sample_fastq(
+            forward_fastq[1].open(), self.forward, exp_indices[4])
+
+        reverse_fastq = [
+            view for path, view in
+            sample.sequences.iter_views(FastqGzFormat)
+            if 'R2_001.fastq' in path.name]
+        self.assertEqual(len(reverse_fastq), 2)
+
+        self._validate_sample_fastq(
+            reverse_fastq[0].open(), self.reverse, exp_indices[3])
+        self._validate_sample_fastq(
+            reverse_fastq[1].open(), self.reverse, exp_indices[4])
+
+    def test_partition_more_partitions_than_samples(self):
+        demux, _ = emp_paired(self.bpsi, self.barcode_map,
+                              golay_error_correction=False)
+
+        with self.assertWarnsRegex(
+                UserWarning, "You have requested a number of.*100.*5.*5"):
+            partition = partition_samples_paired(demux, 100)
+
+        exp_samples_fwd = ('sample1_1_L001_R1_001.fastq.gz',
+                           'sample2_3_L001_R1_001.fastq.gz',
+                           'sample3_2_L001_R1_001.fastq.gz',
+                           'sample4_5_L001_R1_001.fastq.gz',
+                           'sample5_4_L001_R1_001.fastq.gz')
+        exp_samples_rev = ('sample1_1_L001_R2_001.fastq.gz',
+                           'sample2_3_L001_R2_001.fastq.gz',
+                           'sample3_2_L001_R2_001.fastq.gz',
+                           'sample4_5_L001_R2_001.fastq.gz',
+                           'sample5_4_L001_R2_001.fastq.gz')
+        exp_indices = ([0, 5], [2, 4], [1, 3], [7, 10], [6, 8, 9])
+
+        for idx, (id, sample) in enumerate(partition.items()):
+            self.assertEqual(id, f'sample{idx + 1}')
+
+            act_manifest = \
+                list(sample.manifest.view(FastqManifestFormat).open())
+            exp_manifest = \
+                ['sample-id,filename,direction\n',
+                 f'sample{idx + 1},{exp_samples_fwd[idx]},forward\n',
+                 f'sample{idx + 1},{exp_samples_rev[idx]},reverse\n']
+            self._compare_manifests(act_manifest, exp_manifest)
+
+            forward_fastq = [
+                view for path, view in
+                sample.sequences.iter_views(FastqGzFormat)
+                if 'R1_001.fastq' in path.name]
+            self.assertEqual(len(forward_fastq), 1)
+
+            reverse_fastq = [
+                view for path, view in
+                sample.sequences.iter_views(FastqGzFormat)
+                if 'R2_001.fastq' in path.name]
+            self.assertEqual(len(reverse_fastq), 1)
+
+            self._validate_sample_fastq(
+                forward_fastq[0].open(), self.forward, exp_indices[idx])
+            self._validate_sample_fastq(
+                reverse_fastq[0].open(), self.reverse, exp_indices[idx])
+
 
 class SummarizeTests(TestPluginBase):
     package = 'q2_demux.tests'
@@ -999,6 +1116,7 @@ class SummarizeTests(TestPluginBase):
             self.assertTrue(os.path.exists(qual_forward_fp))
             self.assertTrue(os.path.getsize(qual_forward_fp) > 0)
             with open(index_fp, 'r') as fh:
+                assert_no_nans_in_tables(fh)
                 html = fh.read()
                 self.assertIn('<th>Minimum</th>\n      <td>1</td>', html)
                 self.assertIn('<th>Maximum</th>\n      <td>3</td>', html)
@@ -1035,6 +1153,7 @@ class SummarizeTests(TestPluginBase):
             png_fp = os.path.join(output_dir, 'demultiplex-summary.png')
             self.assertFalse(os.path.exists(png_fp))
             with open(index_fp, 'r') as fh:
+                assert_no_nans_in_tables(fh)
                 html = fh.read()
                 self.assertIn('<th>Minimum</th>\n      <td>1</td>', html)
                 self.assertIn('<th>Maximum</th>\n      <td>1</td>', html)
